"""RSS web resource.
"""

from twisted.web import html, resource
from twisted.python import components, log
from twisted.protocols import http

import rsstmpl

True = (1==1)
False = not True

class ChannelView(resource.Resource):
    pageTemplate = rsstmpl.page
    imagerefTemplate = rsstmpl.image_ref
    imagenodeTemplate = rsstmpl.image_node

    def __init__(self, model):
        """
        @type model: L{IChannel}
        """

        resource.Resource.__init__(self)
        self.model = model

    def render(self, request):
        if hasattr(self.model, "getComponent"):
            model = self.model.getComponent(IChannel)
        else:
            model = self.model

        request.setHeader("content-type", "application/rss+xml")

        itemrefs = []
        itemnodes = []
        cached = "maybe"
        for item in model.items:
            if cached and ((item.lastModified is None) or
                           (request.setLastModified(item.lastModified) !=
                            http.CACHED)):
                cached = False

        if cached and model.items:
            log.msg("page is cached!")
            return ''

        for item in model.items:
            itemview = self.itemViewFactory(item)
            itemrefs.append(itemview.view_ref(request, item))
            itemnodes.append(itemview.view_node(request, item))

        if model.image:
            imageref = self.imagerefTemplate % {'image_src': model.image}
            imagenode = self.imagenodeTemplate % {
                'channel_title': model.title,
                'image_src': model.image,
                'title': model.title,
                'web_url': model.webURL,
                }
        else:
            imagenode = imageref = ''

        page = self.pageTemplate % {
            'channel_title': model.title,
            'channel_url': model.channelURL,
            'web_url': model.webURL,
            'channel_desc': html.escape(model.description),
            'image_ref': imageref,
            'item_refs': ''.join(itemrefs),
            'connstatus': "FIXME",
            'image_node': imagenode,
            'item_nodes': ''.join(itemnodes),
            }

        return page


class ItemView:
    itemrefTemplate = rsstmpl.item_ref
    itemnodeTemplate = rsstmpl.item_node

    def __init__(self, model):
        """
        @type model: L{IItem}
        """
        self.model = model

    def view_ref(self, request, model):
        return self.itemrefTemplate % {'item_uri': model.uri}

    def view_node(self, request, model):
        return self.itemnodeTemplate % {
            'item_uri': model.uri,
            'item_title': html.escape(model.title),
            'item_desc': html.escape(model.description),
            }

ChannelView.itemViewFactory = ItemView


class IChannel(components.Interface):
    """
    @ivar title: The channel's title.
    @ivar description: A description of the channel.
    @ivar channelURL: The URL of the RSS channel.
    @ivar webURL: The URL of the site the RSS channel summarizes.

    @ivar image: [optional] The URL of an image associated with the channel.

    @ivar items: An iterable sequence of L{RssItem}s.
    """

class IItem(components.Interface):
    """
    @ivar uri: My URI, which must be unique among all items in my channel.
    @ivar title: My title.
    @ivar description: Description.

    @ivar lastModified: When I was posted.
    @type lastModified: Number; seconds from the epoch.
    """

class Channel:
    __implements__ = (IChannel,)

    title = ""
    description = ""
    webURL = ""
    channelURL = ""

    image = None

    items = None


class Item:
    __implements__ = (IItem,)

    uri = ""
    title = ""
    description = ""

    lastModified = None
