# -*- Python -*-
# $Id: web.py,v 1.35 2004/03/05 06:30:27 acapnotic Exp $

"""Publish commit messages as HTML and RSS web widgets."""

__version__ = '$Revision: 1.35 $'[11:-2]

from twisted.application import internet, strports
from twisted.application.service import MultiService
from twisted.internet import app, defer
from twisted.protocols import http
from twisted.python import components, log, usage
from twisted.spread import pb
from twisted.web import domhelpers, html, resource, server
from twisted.web.woven import model, view, widgets

from cvstoys import interfaces

# siblings
import pb as pbcvs
import rss

# system
import time

# Unresolved formatting issues:
# Where to put the list of files?
# RSS:
#   What does each rss item's link go to?

# TODO:
#  Tests!
#  Put Last-Modified back in.

# General things to factor out into Twisted:
#  RSS publisher

class RecentCommits(pb.Referenceable, components.Componentized):
    __implements__ = (interfaces.INotifieeClient,)

    memoryLength = 15
    avatar = None

    def __init__(self):
        components.Componentized.__init__(self)
        self.commits = []

    def remote_notify(self, root, files, message, user):
        now = time.time()
        self.commits.insert(0, Commit(root, files, message, user, now))
        log.msg("commit from %s to %s" % (user, root))
        if self.memoryLength and (len(self.commits) > self.memoryLength):
            del self.commits[self.memoryLength:]

    def remote_goodbye(self, message):
        log.msg("Disconnecting at server's request: %s" % (message,))
        self.avatar.broker.transport.loseConnection()

    def __len__(self):
        return len(self.commits)


class NotifieeFactory(pbcvs.ReconnectingPBClientFactory):
    def gotPerspective(self, p):
        self.notifiee.avatar = p
        return pbcvs.ReconnectingPBClientFactory.gotPerspective(self, p)

    def startLogin(self, credentials, client):
        self.notifiee = client
        return pbcvs.ReconnectingPBClientFactory.startLogin(
            self, credentials, client)

    def clientConnectionLost(self, connector, reason):
        self.notifiee.avatar = None
        return pbcvs.ReconnectingPBClientFactory.clientConnectionLost(
            self, connector, reason)

class CommitsModel(components.Adapter, model.AttributeModel, rss.Channel):
    __implements__ = (model.AttributeModel.__implements__, rss.IChannel)

    def __init__(self, orig):
        components.Adapter.__init__(self, orig)
        model.AttributeModel.__init__(self, orig)
        self.items = self.commits = self.original.commits

    def getSubmodel(self, request, name):
        try:
            sm = model.adaptToIModel(self.commits[int(name)], self, name)
        except ValueError:
            sm = model.AttributeModel.getSubmodel(self, request, name)
        return sm

    def getPerspective(self):
        return self.original.avatar

    def __len__(self):
        return len(self.original.commits)

components.registerAdapter(CommitsModel, RecentCommits, model.interfaces.IModel)
components.registerAdapter(CommitsModel, RecentCommits, rss.IChannel)

class Commit(model.AttributeModel):
    __implements__ = (rss.IItem, model.interfaces.IModel)

    def initialize(self, root, files, message, user, time):
        self.root = root
        self.files = files
        self.message =  message
        self.user = user
        self.lastModified = time

        self.title, self.description = message.lstrip().split('\n', 1)

        # XXX: kluge
        self.uri = "uri://%s/%s/%s+%x" % (root, user, time, id(self))


class HtmlPage(view.View):
    def wvfactory_freshcvs(self, request, node, model):
        return HtmlRecentCommits(self.model)

    def wvfactory_title(self, request, node, model):
        # XXX: This isn't working at all.  The text widget replaces the
        # node instead of being inserted into it, and the 'title' data
        # on the model isn't being accessed correctly.
        return widgets.Text(model.getData(request))

class HtmlRecentCommits(widgets.Widget):
    """The \"recent commits\" widget.

    """
    def wvfactory_logo(self, request, node, model):
        if not model.image:
            # Eat the entire element.
            domhelpers.clearNode(node)
            return widgets.Text('',clear=True)
        else:
            return widgets.Widget(self.model)

    def wvfactory_Image(self, request, node, model):
        return widgets.Image(model, "logo")

    def wvfactory_link(self, request, node, model):
        w = widgets.Anchor()
        w.setLink(model.webURL)
        w.setText(model.title)
        return w

    def wvfactory_status(self, request, node, model):
        return StatusWidget(model.getPerspective(), "perspective")

    def wvfactory_rssButton(self, request, node, model):
        w = widgets.Anchor()
        w.setLink('rss')
        return w

    def wvfactory_commitList(self, request, node, model):
        return DefinitionList(model)

class StatusWidget(widgets.Widget):
    def setUp(self, request, node, data):
        if data is None:
            self['class'] = "statusOffline"
            content = "off"
        else:
            self['class'] = "statusOnline"
            content = "on<!-- %s -->" % (html.escape(str(data)),)
        domhelpers.clearNode(node)
        self.add(widgets.Text(content, raw=1))

class DefinitionList(widgets.Widget):
    # def setUp(self, request, node, data):
    def generateDOM(self, request, node):
        node = widgets.Widget.generateDOM(self, request, node)
        domhelpers.clearNode(node)
        data = self.getData(request)
        for i in range(len(data)):
            for e in ('definedTerm','definition'):
                newNode = self.getPattern(e, widgets._RAISE)
                widgets.appendModel(newNode, i)
                if not newNode.getAttribute("view"):
                    newNode.setAttribute("view", "commitItem")
                node.appendChild(newNode)
        return node

    def wvfactory_commitItem(self, request, node, model):
        return CommitItem(model)

class CommitItem(widgets.Widget):
    def wvfactory_filelist(self, request, node, data):
        files = ["%s/%s" % (f.pathname, f.filename)
                 for f in data.getData(request)]
        wid = widgets.Text(' '.join(files))
        self.add(wid)
        return wid


#######################


class Options(usage.Options):
    optParameters = [('chost', 'ch', 'localhost'),
                     ('cport', 'cp', 7878),
                     ('cuser', 'cu', 'web'),
                     ('cpass', 'cw', 'notifiee'),
                     # vestigial
                     ('cservice', 'cs', 'cvstoys.notify'),

                     ('webport', 'wp', 8241),

                     ('title', 't', "Recent CVS Activity"),
                     ('description', 'd', ''),
                     ('link', 'l', 'http://twistedmatrix.com/'),
                     ]


class Glue(MultiService):
    pass

def makeService(options):
    from twisted.cred import credentials

    thisServ = Glue()

    # for each notifiee:
    notifiee = RecentCommits()
    factory = NotifieeFactory()
    mycreds = []
    #if options['cuser']:
    mycreds.append(credentials.UsernamePassword(options['cuser'],
                                                options['cpass']))
    #else:
    # mycreds.append(credentials.Anonymous())
    factory.startLogin(mycreds[0], notifiee)
    # FIXME: no strports.client (Twisted issue 511)
    c = internet.TCPClient(options['chost'], int(options['cport']),
                           factory)
    c.setServiceParent(thisServ)
    # next notifiee

    m = notifiee.getComponent(rss.IChannel)
    m.title = options['title']
    m.description = options['description']
    m.webURL = options['link']
    m2 = notifiee.getComponent(model.interfaces.IModel)
    assert m is m2
    root = resource.Resource()
    root.putChild('', HtmlPage(notifiee, templateFile="recent_template.html"))
    root.putChild('rss', rss.ChannelView(notifiee))

    webserver = strports.service(str(options['webport']), server.Site(root))
    webserver.setServiceParent(thisServ)

    return thisServ
