#$Id: loginfo.py,v 1.12 2003/09/07 21:33:34 acapnotic Exp $

"""CVS uses me to notify the daemon of changes.

Add this line to the 'loginfo' file in the 'CVSROOT' module.

DEFAULT loginfo %{sVv}
"""

__version__ = '$Revision: 1.12 $'[11:-2]

from twisted.python.compat import *
from twisted.internet import reactor
from twisted.python import usage, log
from twisted.spread import pb
import os, string, sys
from os import path

from common import VersionedPatch, Directory, SOCKETNAME
import pidutils


class MyOptions(usage.Options):
    optFlags = [("debug", "d", "when not really running under CVS"),
                ("reconfigure","r",("instruct the server to re-load"
                                    " the configuration file")),
                ("commitinfo", "c", "when running from commitinfo"),
                ("loginfo", "l", "when running from loginfo")]

    optParameters = [("pipe", "p", path.join(os.environ.get('CVSROOT',
                                                            os.getcwd()),
                                             'CVSROOT', SOCKETNAME),
                      "The pipe to talk to freshcvs on.")]

    # TODO: Add host, port (or socket-filename)

    def postOptions(self):
        n = len(filter(None, [self.opts['reconfigure'],
                              self.opts['commitinfo'],
                              self.opts['loginfo']]))
        if n > 1:
            raise usage.UsageError("reconfigure, commitinfo, and loginfo are"
                                   " mutually exclusive.  Please pick one.")
        if n == 0:
            # TODO: Check argv[0].
            self.opts['loginfo'] = True


    def parseArgs(self, *args):
        if self.opts['reconfigure']:
            return
        else:
            pathname, files = args[0], args[1:]

        if self.opts['commitinfo']:
            self.pathname = pathname
            return

        if not files and ' ' in pathname:
            # <glyph> shellquoting's a bitch
            pathname, files = string.split(pathname, ' ', 1)
            files = string.split(files)
        self.pathname = pathname

        if files and files[0] == '-':
            msg = string.join(files, ' ')
            # Hooray for special cases!
            if msg == "- New directory":
                self.pathname = path.dirname(pathname)
                file = path.basename(pathname)
                self.files = [Directory(file)]
                return
            elif msg == "- Imported sources":
                # XXX: Do something useful here!
                self.files = []
                return

        def parseFileArg(arg):
            """Parses loginfo's %{sVv} string to (file, (verson, version))

            Returns a tuple with two members, the filename and a tuple
            of (oldversion, newversion).

            In the event that the argument does not appear to be in the
            expected format, a warning is issued and None is returned.
            """

            split = string.split(arg,",")
            if len(split) < 3:
                log.msg("Argument '%s' does not appear"
                        " to be in %%{sVv} format." % (arg,))
                return None

            # Assume the last two items are the versions --
            # if there are more, pretend that the first few were
            # created by a filename with a comma in it.
            file = string.join(split[:-2], ',')
            revisions = split[-2:]

            for i in range(len(revisions)):
                if revisions[i] == 'NONE':
                    revisions[i] = (None,)
                else:
                    try:
                        parts = string.split(revisions[i], ".")
                        revisions[i] = tuple(map(int, parts))
                    except (TypeError, ValueError), e:
                        # Can't be converted to int
                        log.msg("version #%d '%s' in '%s'"
                                " is in an unexpected format: %s"
                                % (i+1, revisions[i], arg, str(e)))
                        revisions[i] = (revisions[i],)

            return VersionedPatch(file, revisions[0], revisions[1])

        # At the moment, loginfo cares about the list of files,
        # while commitinfo is satisfied with just the pathname.
        if not self.opts['commitinfo']:
            self.files = filter(None, map(parseFileArg, files))

def cb_loginfo(object, cvs_pid, root, directory, files, user, message):
    d = object.callRemote('loginfo', cvs_pid, root, directory, files,
                          user, message)
    d.addCallbacks(cb_stop, eb_stop)

def cb_commitinfo(object, cvs_pid, root, directory):
    d = object.callRemote("hasDirectory", cvs_pid, root, directory)
    d.addCallbacks(cb_stop, eb_stop)

def cb_reconfigure(object, root):
    d = object.callRemote('reconfigure', root)
    # No reply is required for reconfigure, but we'll wait for a callback
    # before stopping out of politeness.
    d.addCallbacks(cb_stop, eb_stop)

def cb_stop(*unused):
    """Shut down cleanly."""
    reactor.stop()

def eb_stop(reason):
    """Shut down after logging a failure."""
    log.err(reason)
    reactor.stop()

def aint(reason):
    sys.stderr.write("fresher didn't connect: %s\n" % (reason,))
    reactor.crash() # os._exit(1) # XXX reactor.stop()

def main_func(config, check_environment=True):
    """I gather information from the environment and make the PB call.

    set check_environment to False for debugging purposes only.
    """

    try:
        cvsroot = os.environ['CVSROOT']
    except KeyError:
        if check_environment:
            raise RuntimeError, "$CVSROOT is not set in my environment"
        else:
            cvsroot = os.getcwd()

    cvsuser = os.getenv("CVS_USER")\
              or os.getenv("LOGNAME")\
              or os.getenv("USER")

    (host, port) = ("unix", config.opts['pipe'])

    if config.opts['reconfigure']:
        cfactory = pb.PBClientFactory()
        reactor.connectUNIX(config.opts['pipe'], cfactory)
        d = cfactory.getRootObject()
        d.addCallbacks(cb_reconfigure, aint, callbackArgs=(cvsroot,))
        return reactor.run()

    cvs_pid = os.getpgrp()
    # cvs_pid = pidutils.findParentNamed('cvs')

    if (cvs_pid is None) and check_environment:
        raise RuntimeError, "Cannot find a cvs parent process"

    if config.opts['loginfo']:
        if not sys.stdin.isatty():
            message = sys.stdin.readlines()
            try:
                lnum = message.index('Log Message:\n') + 1
            except ValueError:
                lnum = 0

            del message[:lnum]
            message = string.join(message, '')
        else:
            message = None

        cfactory = pb.PBClientFactory()
        reactor.connectUNIX(config.opts['pipe'], cfactory)
        d = cfactory.getRootObject()
        d.addCallbacks(cb_loginfo, aint,
                       callbackArgs=(cvs_pid, cvsroot, config.pathname,
                                     config.files, cvsuser, message))
    elif config.opts['commitinfo']:
        cfactory = pb.PBClientFactory()
        reactor.connectUNIX(config.opts['pipe'], cfactory)
        d = cfactory.getRootObject()
        d.addCallbacks(cb_commitinfo, aint,
                       callbackArgs=(cvs_pid, cvsroot, config.pathname))

    # BUG: Eek.  getObjectAt may call the failure callback before
    # reactor.run() happens.  That's supposed to stop, but we can't
    # stop before run.
    # XXX: Is this still true?  I'm not sure it is since Deferreds
    # happened for the callbacks.
    return reactor.run()


def main():
    config = MyOptions()
    try:
        config.parseOptions()
    except:
        print str(config)
        raise

    e = main_func(config, not config.opts['debug'])
    sys.exit(e)
