# $Id: setup.py,v 1.13 2003/04/02 03:59:01 acapnotic Exp $

from __future__ import nested_scopes
import errno, os, shutil, sys, tempfile, traceback
from os import path

from twisted.python import usage, util

try:
    import readline
except:
    pass

True=(1==1)
False=not True

scriptDirectory = "/usr/local/bin"

defaultConfig = """# -*- Python -*-
# See the documentation for cvstoys.config and cvstoys.actions.
# Argument order is (repository, path, file, action).

# Change address and uncomment to get e-mail notification.
#
# mail = ConfigurationSet([
#     (None, None, None, MailNotification([\"cvs-commits@localhost\"])),
#     ])

# This will log every commit in twistd.log.
debug = ConfigurationSet([
    (None, None, None, DebugAction(\"Default debug log\")),
    ])
"""

class Options(usage.Options):
    optParameters = [('root', 'd', None, "Path to the CVS repository."),
                     ('cfg', 'f', None, "File to install as freshCfg."),
                     ('pipe', 'p', None,
                      "Where to put the listening unix socket."),
                     ]

    def opt_manhole(self, password):
        """Password to the manhole. - will read from stdin or prompt.
        """
        # If standard input is a terminal, I prompt for a password and
        # confirm it.  Otherwise, I use the first line from standard
        # input, stripping off a trailing newline if there is one.
        if password in ('-',):
            self.opts['manhole'] = util.getPassword(confirm=True)
        else:
            self.opts['manhole'] = password

    def postOptions(self):
        if self.opts['cfg']:
            cfgfilename = path.abspath(self.opts['cfg'])
            cfgfile = open(cfgfilename, 'r')
            # Check to make sure that the file is syntatically valid.
            try:
                try:
                    compile(cfgfile.read(), cfgfilename, 'exec')
                finally:
                    cfgfile.close()
            except SyntaxError:
                warn("Configuration file %s does not have valid Python syntax."
                     % (cfgfilename,))
                traceback.print_exc()
                sys.exit(1)
            self.opts['cfg'] = cfgfilename


class SubprocessError(Exception):
    """Something went wrong in a subprocess."""
    def __init__(self, name, exitcode):
        Exception.__init__(self, name, exitcode)
        self.name = name
        self.exitcode = exitcode

def system(s):
    exitcode = os.system(s)
    if exitcode:
        raise SubprocessError(s, exitcode)
    return exitcode

def say(s):
    print s

def warn(s):
    sys.stderr.write(s + '\n')

def main():
    origDir = os.getcwd()
    config = Options()
    try:
        config.parseOptions()
    except:
        print str(config)
        raise

    repository = config.opts['root'] or chooseRepository(autodetect=True)
    root = None

    while not root:
        try:
            permissionCheck(repository)
            permissionCheck(path.join(repository, "CVSROOT"))
        except IOError, e:
            warn(str(e))
            repository = chooseRepository()
        else:
            root = path.join(repository, "CVSROOT")

    say("Installing in %s" % (repr(root,)))
    cvsdance(repository, config.opts['pipe'], config.opts['cfg'])
    os.chdir(origDir)
    say("Making freshcvs.tap for you.")

    manpass = config.opts.get('manhole')

    if manpass is None:
        say("""\
I recommend including a manhole in your freshcvs service, should
it need servicing.  If you don't want this, enter a blank password here
(by just hitting <Enter> twice), and no manhole will be created.""")

        manpass = util.getPassword(confirm=True)
        if manpass:
            say("""\
You may log in to the twisted.manhole service with username 'manhole' and
the password you just specified.""")
        else:
            manpass = None
            say("Ok, no manhole for you.")

    if config.opts['pipe']:
        pipeStr = '--pipe %s' % (config.opts['pipe'],)
    else:
        pipeStr = ''

    if not manpass:
        system("mktap freshcvs %(pipe)s" % {'pipe': pipeStr,})
    else:
        child_in, child_out = os.popen4("mktap freshcvs %(pipe)s --manhole -"
                                        % {'pipe': pipeStr,})
        child_in.write(manpass + '\n')
        child_in.close()
        say(child_out.read())
        child_out.close()

    say("Done!")

    say("""\
To start your fresh CVS daemon: twistd -f freshcvs.tap
(See twistd(1) for options on where to write the log and pidfile.)
Your current configuration will only log commits to twistd.log.  To
change it, check out the CVSROOT module from the %(repository)s
repository, and edit the file %(cfgfile)s.
For the a new configuration to take effect, you must check in %(cfgfile)s.
""" % {'repository': repository,
       'cfgfile': 'freshCfg',
       'bin': scriptDirectory, })

def cvsdance(repository, pipe=None, cfgfile=None):
    scratch = tempfile.mktemp('-inst')
    os.mkdir(scratch)
    os.chdir(scratch)
    say("Working in scratch directory %s." % (scratch,))

    os.environ['CVSROOT'] = repository
    say("Checking out CVS administrative directory.")
    system("cvs -Q checkout CVSROOT")
    os.chdir("CVSROOT")

    if path.exists("freshCfg"):
        say("There's already a freshCfg file here!"
            "  Aborting automatic set-up.")
        return

    if cfgfile:
        say("Installing %s as configuration file." % (cfgfile,))
        shutil.copy(cfgfile, "freshCfg")
    else:
        say("Writing a default configuration file.")
        cfg = open("freshCfg", "w")
        try:
            cfg.write(defaultConfig)
        finally:
            cfg.close()

    system("cvs -Q add -m 'CVSToys configuration' freshCfg")

    say("Adding configuration file to list of administrative files.")
    needAdd = not path.exists("checkoutlist")
    checkoutlist = open("checkoutlist", 'a')
    try:
        checkoutlist.write("freshCfg\n")
    finally:
        checkoutlist.close()

    if needAdd:
        system("cvs -Q add checkoutlist")

    say("Adding to the list of programs to call with log messages.")

    if pipe:
        pipeStr = '--pipe %s' % (pipe,)
    else:
        pipeStr = ''

    needAdd = not path.exists("loginfo")

    loginfo = open("loginfo", "a")
    try:
        loginfo.write("ALL %(bin)s/loginfo %(pipe)s %%{sVv}\n"
                      % {'pipe': pipeStr,
                         'bin': scriptDirectory})
    finally:
        loginfo.close()

    if needAdd:
        system("cvs -Q add loginfo")

    say("Adding to the list of programs to call pre-commit.")
    needAdd = not path.exists("commitinfo")

    commitinfo = open("commitinfo", "a")
    try:
        commitinfo.write("ALL %(bin)s/loginfo %(pipe)s --commitinfo\n"
                      % {'pipe': pipeStr,
                         'bin': scriptDirectory})
    finally:
        commitinfo.close()

    if needAdd:
        system("cvs -Q add commitinfo")

    say("Committing changes to the repository.")
    system("cvs -Q commit -m 'Install CVSToys'")

def chooseRepository(autodetect=False):
    choices = detectRepository()
    if autodetect and (len(choices) == 1):
        repository = choices[0]
        say("Using detected value %s for CVSROOT." % (repr(repository)))
        return repository
    elif not choices:
        return promptForRepository()
    else:
        say("Install in which CVS repository:")
        i = 1
        for c in choices:
            say("%d) %s" % (i, c))
        say("0) None of the above.")
        say("")
        while 1:
            selection = raw_input("Pick one: #")
            try:
                index = int(selection)
            except ValueError:
                say("%s isn't a number." % (repr(selection,)))
                continue
            if index == 0:
                return promptForRepository()
            try:
                return choices[index-1]
            except IndexError:
                say("%d isn't on the list." % (index,))
                continue


def promptForRepository():
    rough = raw_input("Local CVS repository directory: ")
    return path.expanduser(path.expandvars(rough))

def detectRepository():
    canidates = []
    if os.environ.has_key('CVSROOT'):
        # $CVSROOT is set.
        root = isLocal(os.environ['CVSROOT'])
        if not root:
            warn("$CVSROOT %s is not a local repository." %
                 (repr(os.environ['CVSROOT'],)))
        else:
            canidates.append(root)

    if path.isdir('CVS') and path.isfile(path.join("CVS", "Root")):
        # Are we in a CVS working directory?
        try:
            root = open(path.join("CVS", "Root"), 'r').read().strip()
        except IOError, e:
            warn("I thought I might be in a working directory, but couldn't"
                 " read CVS/Root: %s" % (e,))
        else:
            lroot = isLocal(root)
            if not lroot:
                warn("CVS/Root does not point to a local repository: %s" %
                     (repr(root,)))
            else:
                canidates.append(lroot)

    if path.isdir('CVSROOT'):
        maybe = isaCVSROOT('CVSROOT')
        if maybe:
            canidates.append(os.getcwd())
        else:
            warn("There's a directory called CVSROOT here, but it doesn't"
                 " look much like a repository.")

    if isaCVSROOT():
        canidates.append(path.abspath(".."))

    nodupes = []
    for c in canidates:
        if c not in nodupes:
            nodupes.append(c)

    return nodupes


def isLocal(rootstring):
    if rootstring.startswith('/'):
        return rootstring
    elif rootstring.startswith(":local:"):
        return rootstring[7:]
    else:
        return False

def isaCVSROOT(dir=None):
    if dir is None:
        dir = os.getcwd()
    def isThere(p):
        return path.exists(p) and path.exists(p + ',v')
    return not not filter(lambda f: isThere(path.join(dir, f)),
                          ['config','cvswrappers','loginfo','rcsinfo'])

def permissionCheck(dir):
    if not path.exists(dir):
        raise IOError, (errno.ENOENT, os.strerror(errno.ENOENT),
                        dir)
    if not path.isdir(dir):
        raise IOError, (errno.ENOTDIR, os.strerror(errno.ENOTDIR),
                        dir)
    if not os.access(dir, os.R_OK | os.W_OK | os.X_OK):
        raise IOError, (errno.EACCES, os.strerror(errno.EACCES),
                        dir)

if __name__ == '__main__':
    main()
