# -*- Python -*-
# $Id: util.py,v 1.4 2003/04/20 00:35:44 acapnotic Exp $
#
# By Kevin Turner <acapnotic@twistedmatrix.com>, October 2001
# <license common_name="BSD license">
# Redistribution and use in source and binary forms, with
# or without modification, are permitted provided that the
# following conditions are met:
#
#    1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    2. Redistributions in binary form must reproduce the
#    above copyright notice, this list of conditions and
#    the following disclaimer in the documentation and/or
#    other materials provided with the distribution.
#
#    3. The name of the author may not be used to endorse
#    or promote products derived from this software without
#    specific prior written permission.
#
# <!-- I am not happy with the person who makes it necessary
# to put something like the paragraph which follows on every
# header file and coffee cup. -->
#
# This software is provided by the author "as is" and any express
# or implied warranties, including, but not limited to, the implied
# warranties of merchantability and fitness for a particular purpose are
# disclaimed. In no event shall the author be liable for any direct,
# indirect, incidental, special, exemplary, or consequential damages
# (including, but not limited to, procurement of substitute goods or
# services; loss of use, data, or profits; or business interruption)
# however caused and on any theory of liability, whether in contract,
# strict liability, or tort (including negligence or otherwise) arising
# in any way out of the use of this software, even if advised of the
# possibility of such damage.
# </license>

"""Support functions used by notification actions or clients."""

import string
from os import path

import common

def viewcvs_urls(base, cvsroot, files):
    """Make references to a ViewCVS web service.

    @param cvsroot: As it is passed it to the ViewCVS CGI.  Its value
    may vary depending on your configuration.
    @type cvsroot: string

    @type files: list of VersionedPatch

    @returns: a list of URL strings.
    """
    viewUrls = []
    for f in files:
        if '/' in f.pathname:
            # NOTE: this replaces the supplied cvsroot with the module name.
            # XXX: Change the documentation to reflect this!
            cvsroot, filename = f.pathname.split('/', 1)
            filename = path.join(filename, f.filename)
        else:
            cvsroot = f.pathname
            filename = f.filename

        if isinstance(f, common.VersionedPatch):
            revs = [f.r1, f.r2]
            for i in (0,1):
                if revs[i] is None:
                    revs[i] = 'NONE'
                else:
                    revs[i] = string.join(map(str, revs[i]), '.')
            if f.r1 == (None,):
                # New file.
                args = [
                    'rev=%s' % revs[1],
                    'content-type=text/vnd.viewcvs-markup'
                    ]
            else:
                # Diff
                filename = filename + '.diff'
                args = [
                    "r1=text",
                    "tr1=%s" % revs[0],
                    "r2=text",
                    "tr2=%s" % revs[1]
                    ]
        else:
            # e.g. a directory or new import
            args = []

        if cvsroot:
            args.append("cvsroot=%s" % (cvsroot,))

        url = "%(base)s%(file)s?%(args)s" % {
            'base': base,
            'file': filename,
            'args': string.join(args,'&')
            }

        viewUrls.append(url)

    return viewUrls


### Local Variables:
### elide-head-headers-to-hide:(("<license" . "</license>"))
### End:
